@@ -3,7 +3,7 @@ package Jifty::DBI::Filter::DateTime;
 use warnings;
 use strict;
 
-use base qw|Jifty::DBI::Filter|;
+use base qw|Jifty::DBI::Filter Class::Data::Inheritable|;
 use DateTime                  ();
 use DateTime::Format::ISO8601 ();
 use DateTime::Format::Strptime ();
@@ -11,7 +11,7 @@ use Carp ();
 
 use constant _time_zone => 'UTC';
 use constant _strptime  => '%Y-%m-%d %H:%M:%S';
-
+use constant _parser    => DateTime::Format::ISO8601->new();
 
 =head1 NAME
 
@@ -26,6 +26,27 @@ minute, and second information is discarded when encoding.
 Both input and output will always be coerced into UTC (or, in the case of
 Dates, the Floating timezone) for consistency.
 
+=head2 formatter
+
+This is an instance of the DateTime::Format object used for inflating the
+string in the database to a DateTime object. By default it is a
+L<DateTime::Format::Strptime> object that uses the C<_strptime> method as its
+pattern.
+
+You can use the _formatter classdata storage as a cache so you don't need
+to re-instantiate your format object every C<decode>.
+
+=cut
+
+__PACKAGE__->mk_classdata("_formatter");
+sub formatter {
+    my $self = shift;
+    unless ($self->_formatter) {
+         $self->_formatter(DateTime::Format::Strptime->new(pattern => $self->_strptime));
+    }
+    return $self->_formatter;
+}
+
 =head2 encode
 
 If value is DateTime object then converts it into ISO format
@@ -81,7 +102,7 @@ sub decode {
 
     my $str = join('T', split ' ', $$value_ref, 2);
     my $dt;
-    eval { $dt  = DateTime::Format::ISO8601->parse_datetime($str) };
+    eval { $dt  = $self->_parser->parse_datetime($str) };
 
     if ($@) { # if datetime can't decode this, scream loudly with a useful error message
         Carp::cluck($@);
@@ -92,7 +113,7 @@ sub decode {
 	my $tz = $self->_time_zone;
 	$dt->set_time_zone($tz) if $tz;
 
-        $dt->set_formatter(DateTime::Format::Strptime->new(pattern => $self->_strptime));
+        $dt->set_formatter($self->formatter);
         $$value_ref = $dt;
     } else {
         return;
